# Generated by Django. Initial migration for BankAccount, Payee, and Check models.
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="BankAccount",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("street_1", models.CharField(blank=True, max_length=255)),
                ("street_2", models.CharField(blank=True, max_length=255)),
                ("city", models.CharField(blank=True, max_length=255)),
                ("phone", models.CharField(blank=True, max_length=50)),
                ("routing_number", models.CharField(max_length=20)),
                ("account_number", models.CharField(max_length=20)),
            ],
        ),
        migrations.CreateModel(
            name="Payee",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("street_1", models.CharField(blank=True, max_length=255)),
                ("street_2", models.CharField(blank=True, max_length=255)),
                ("city", models.CharField(blank=True, max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name="Check",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("payor_name", models.CharField(max_length=255)),
                ("payor_street_1", models.CharField(blank=True, max_length=255)),
                ("payor_street_2", models.CharField(blank=True, max_length=255)),
                ("payor_city", models.CharField(blank=True, max_length=255)),
                ("payor_phone", models.CharField(blank=True, max_length=50)),
                ("payor_account_number", models.CharField(blank=True, max_length=20)),
                ("check_number", models.PositiveIntegerField()),
                ("amount", models.DecimalField(decimal_places=2, max_digits=10)),
                ("memo", models.CharField(blank=True, max_length=255)),
                ("invoice", models.CharField(blank=True, max_length=255)),
                ("check_date", models.DateField(default=django.utils.timezone.now)),
                ("pdf", models.FileField(blank=True, null=True, upload_to="checks/")),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("updated", models.DateTimeField(auto_now=True)),
                (
                    "bank_account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="papercheck.BankAccount",
                    ),
                ),
                (
                    "payee",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="papercheck.Payee",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 5.2.4 on 2025-07-21 17:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("papercheck", "0002_alter_bankaccount_id_alter_check_id_alter_payee_id"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="check",
            name="payor_account_number",
        ),
        migrations.RemoveField(
            model_name="check",
            name="payor_city",
        ),
        migrations.RemoveField(
            model_name="check",
            name="payor_name",
        ),
        migrations.RemoveField(
            model_name="check",
            name="payor_phone",
        ),
        migrations.RemoveField(
            model_name="check",
            name="payor_street_1",
        ),
        migrations.RemoveField(
            model_name="check",
            name="payor_street_2",
        ),
        migrations.AddField(
            model_name="bankaccount",
            name="payor_city",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="bankaccount",
            name="payor_name",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="bankaccount",
            name="payor_phone",
            field=models.CharField(blank=True, max_length=50),
        ),
        migrations.AddField(
            model_name="bankaccount",
            name="payor_street_1",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="bankaccount",
            name="payor_street_2",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="payee",
            name="payor_account_number",
            field=models.CharField(blank=True, max_length=20),
        ),
    ]



# Generated by Django 5.2.4 on 2025-07-21 17:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("papercheck", "0003_remove_check_payor_account_number_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="check",
            name="check_number",
            field=models.PositiveIntegerField(editable=False, unique=True),
        ),
    ]



# Generated by Django 5.2.4 on 2025-07-21 17:59

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "papercheck",
            "0006_rename_descriptive_name_bankaccount_account_nickname_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
            ],
        ),
        migrations.AddField(
            model_name="check",
            name="category",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="papercheck.category",
            ),
        ),
    ]



# Generated by Django 5.2.4 on 2025-07-21 17:47

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("papercheck", "0005_rename_name_bankaccount_formal_name_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="bankaccount",
            old_name="descriptive_name",
            new_name="account_nickname",
        ),
        migrations.RenameField(
            model_name="bankaccount",
            old_name="formal_name",
            new_name="institution_name",
        ),
    ]



# Generated by Django 5.2.4 on 2025-07-21 18:41

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("papercheck", "0008_check_tags"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="category",
            options={"ordering": ["name"], "verbose_name_plural": "Categories"},
        ),
    ]



# Generated by Django 5.2.4 on 2025-07-21 18:19

import taggit.managers
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("papercheck", "0007_category_check_category"),
        (
            "taggit",
            "0006_rename_taggeditem_content_type_object_id_taggit_tagg_content_8fc721_idx",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="check",
            name="tags",
            field=taggit.managers.TaggableManager(
                blank=True,
                help_text="A comma-separated list of tags.",
                through="taggit.TaggedItem",
                to="taggit.Tag",
                verbose_name="Tags",
            ),
        ),
    ]



from django.contrib import admin
from .models import BankAccount, Payee, Check, Category
from django.http import HttpResponse
import csv
from django.contrib.admin import DateFieldListFilter


@admin.register(Category)
class CategoryAdmin(admin.ModelAdmin):
    list_display = ("name",)


@admin.register(BankAccount)
class BankAccountAdmin(admin.ModelAdmin):
    list_display = (
        "account_nickname",
        "institution_name",
        "account_number",
        "routing_number",
        "city",
    )


@admin.register(Payee)
class PayeeAdmin(admin.ModelAdmin):
    list_display = ("name", "city")


@admin.register(Check)
class CheckAdmin(admin.ModelAdmin):
    list_display = ("check_number", "payee", "bank_account", "amount", "check_date")
    readonly_fields = ("check_number", "pdf")
    search_fields = ("check_number", "payee__name", "bank_account__name")
    list_filter = ("bank_account", "payee", ("check_date", DateFieldListFilter))
    actions = ["export_to_monarchmoney_csv"]

    def export_to_monarchmoney_csv(self, request, queryset):
        response = HttpResponse(content_type="text/csv")
        response["Content-Disposition"] = "attachment; filename=checks_monarchmoney.csv"
        writer = csv.writer(response)
        writer.writerow(
            [
                "Date",
                "Merchant",
                "Category",
                "Account",
                "Original Statement",
                "Notes",
                "Amount",
                "Tags",
            ]
        )
        for check in queryset:
            date = check.check_date.strftime("%Y-%m-%d")
            merchant = check.payee.name
            category = check.category.name if check.category else ""
            account = (
                check.bank_account.account_nickname
                or check.bank_account.institution_name
            )
            original_statement = (
                f"{check.payee.name} - {check.payee.payor_account_number}"
            )
            notes = f"{check.check_number} - {check.payee.name} - {check.payee.payor_account_number} - {check.memo} - {check.invoice}"
            amount = f"{check.amount:.2f}"
            tags = (
                ", ".join(tag.name for tag in check.tags.all())
                if hasattr(check, "tags")
                else ""
            )
            writer.writerow(
                [
                    date,
                    merchant,
                    category,
                    account,
                    original_statement,
                    notes,
                    amount,
                    tags,
                ]
            )
        return response

    export_to_monarchmoney_csv.short_description = (
        "Export selected checks to MonarchMoney CSV"
    )



# Generated by Django 5.2.4 on 2025-07-21 17:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("papercheck", "0004_alter_check_check_number"),
    ]

    operations = [
        migrations.RenameField(
            model_name="bankaccount",
            old_name="name",
            new_name="formal_name",
        ),
        migrations.AddField(
            model_name="bankaccount",
            name="descriptive_name",
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import letter
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from io import BytesIO
from num2words import num2words
import os

# Register the MICR font (relative to this file)
micr_font_path = os.path.join(os.path.dirname(__file__), "micr-encoding/micrenc.ttf")
if not os.path.isfile(micr_font_path):
    raise FileNotFoundError(f"MICR font file not found at {micr_font_path}")
pdfmetrics.registerFont(TTFont("MICR", micr_font_path))


def generate_check_pdf(check):
    """
    Generate a PDF for a Check instance and return bytes.
    """
    buf = BytesIO()
    c = canvas.Canvas(buf, pagesize=letter)
    width, height = letter

    # Bank info
    bank = check.bank_account
    bank_name = bank.institution_name
    bank_street_1 = bank.street_1
    bank_street_2 = bank.street_2
    bank_city = bank.city
    bank_phone = bank.phone
    ACH_ROUTING_NUMBER = bank.routing_number
    ACH_ACCOUNT_NUMBER = bank.account_number

    # Payor info (now on BankAccount)
    payor_name = bank.payor_name
    payor_street_1 = bank.payor_street_1
    payor_street_2 = bank.payor_street_2
    payor_city = bank.payor_city
    payor_phone = bank.payor_phone

    # Payee info
    payee = check.payee
    payee_name = payee.name
    payee_street_1 = payee.street_1
    payee_street_2 = payee.street_2
    payee_city = payee.city
    payor_account_number = payee.payor_account_number

    # Check info
    check_number = str(check.check_number)
    amount = f"{check.amount:.2f}"
    amount_split = amount.split(".")
    amount_words = (
        f"***{num2words(amount_split[0]).title()} and {amount_split[1]}/100***"
    )
    memo = check.memo
    invoice = check.invoice
    check_date = check.check_date.strftime("%m/%d/%Y")

    # Margins
    left_margin = 0.25 * 72
    top_margin = height - 0.25 * 72
    right_margin = width - 0.25 * 72
    payee_alignment = left_margin + 70

    micr_line = f"C{ACH_ROUTING_NUMBER}C A{ACH_ACCOUNT_NUMBER}A {check_number}D"

    # === Payor Information ===
    c.setFont("Helvetica-Bold", 15)
    c.drawString(left_margin, top_margin - 10, payor_name)
    c.setFont("Helvetica", 10)
    payer_line_change = 12
    payor_address_y = top_margin - 22
    if payor_street_1:
        c.drawString(left_margin, payor_address_y, payor_street_1)
        payor_address_y -= payer_line_change
    if payor_street_2:
        c.drawString(left_margin, payor_address_y, payor_street_2)
        payor_address_y -= payer_line_change
    if payor_city:
        c.drawString(left_margin, payor_address_y, payor_city)
        payor_address_y -= payer_line_change
    if payor_phone:
        c.drawString(left_margin, payor_address_y, payor_phone)
        payor_address_y -= payer_line_change

    # === Bank Information ===
    bank_address_left_margin = left_margin + 250
    c.setFont("Helvetica-Bold", 10)
    c.drawString(bank_address_left_margin, top_margin - 10, bank_name)
    c.setFont("Helvetica", 8)
    bank_address_y = top_margin - 21
    bank_address_line_change = 10
    if bank_street_1:
        c.drawString(bank_address_left_margin, bank_address_y, bank_street_1)
        bank_address_y -= bank_address_line_change
    if bank_street_2:
        c.drawString(bank_address_left_margin, bank_address_y, bank_street_2)
        bank_address_y -= bank_address_line_change
    if bank_city:
        c.drawString(bank_address_left_margin, bank_address_y, bank_city)
        bank_address_y -= bank_address_line_change
    if bank_phone:
        c.drawString(bank_address_left_margin, bank_address_y, bank_phone)
        bank_address_y -= bank_address_line_change

    # === Pay ===
    c.setFont("Helvetica-Bold", 10)
    c.drawString(left_margin, top_margin - 70, "Pay")
    c.setFont("Helvetica", 10)
    c.drawString(payee_alignment, top_margin - 70, amount_words)

    # === PAY TO THE ORDER OF ===
    c.setFont("Helvetica", 10)
    c.drawString(left_margin, top_margin - 95, "To the")
    c.drawString(left_margin, top_margin - 105, "Order of:")
    c.drawString(payee_alignment, top_margin - 95, payee_name)

    # === PAYEE ADDRESS ===
    address_y = top_margin - 110
    if payee_street_1:
        c.drawString(payee_alignment, address_y, payee_street_1)
        address_y -= 12
    if payee_street_2:
        c.drawString(payee_alignment, address_y, payee_street_2)
        address_y -= 12
    if payee_city:
        c.drawString(payee_alignment, address_y, payee_city)
        address_y -= 12

    # === CHECK NUMBER ===
    c.setFont("Helvetica-Bold", 12)
    c.drawString(right_margin - 35, top_margin - 5, check_number)

    # === DATE FIELD (optional line) ===
    c.line(left_margin + 450, top_margin - 40, right_margin - 10, top_margin - 40)
    c.drawString(left_margin + 450, top_margin - 35, "Date")
    c.setFont("Helvetica", 10)
    c.drawString(left_margin + 485, top_margin - 35, check_date)

    # === AMOUNT BOX ===
    c.rect(left_margin + 450, top_margin - 75, 115, 20)
    c.setFont("Helvetica-Bold", 14)
    c.drawString(left_margin + 465, top_margin - 70, f"***{amount}***")

    # === MEMO / DESCRIPTION ===
    c.line(left_margin, top_margin - 160, left_margin + 250, top_margin - 160)
    c.setFont("Helvetica", 10)
    c.drawString(left_margin, top_margin - 170, "Memo")  # placeholder
    c.drawString(left_margin, top_margin - 155, memo)

    # === SIGNATURE LINE ===
    c.line(left_margin + 375, top_margin - 160, right_margin - 10, top_margin - 160)
    c.setFont("Helvetica", 10)
    c.drawString(left_margin + 375, top_margin - 170, "Signature")

    # === MICR LINE ===
    c.setFont("MICR", 16)
    c.drawString(left_margin + 125, top_margin - 200, micr_line)

    # === Stub 1 Information ===
    c.setFont("Helvetica-Bold", 12)
    stub_1_y = top_margin - 300
    stub_1_y_offset = 25

    c.drawString(left_margin + 25, stub_1_y, f"Check #{check_number}")
    c.drawString(left_margin + 450, stub_1_y, f"${amount}")
    stub_1_y -= stub_1_y_offset * 1.2
    c.setFont("Helvetica", 12)
    c.drawString(left_margin + 25, stub_1_y, payee_name)
    stub_1_y -= stub_1_y_offset * 1.2
    c.drawString(left_margin + 25, stub_1_y, memo)
    c.setFont("Helvetica", 12)
    stub_1_y -= stub_1_y_offset
    if payor_account_number:
        c.drawString(left_margin + 25, stub_1_y, f"Account: {payor_account_number}")
    stub_1_y -= stub_1_y_offset
    if invoice:
        c.drawString(left_margin + 25, stub_1_y, f"Invoice: {invoice}")
    stub_1_y -= stub_1_y_offset * 2
    c.drawString(left_margin + 25, stub_1_y, f"From: {payor_name}")
    c.drawString(left_margin + 450, stub_1_y, check_date)

    # === Stub 2 Information ===
    stub_2_y = top_margin - 600
    stub_2_y_offset = 15
    c.setFont("Helvetica-Bold", 12)
    c.drawString(left_margin + 25, stub_2_y, f"Check #{check_number}")
    c.drawString(left_margin + 450, stub_2_y, f"${amount}")
    stub_2_y -= stub_2_y_offset * 1.5
    c.setFont("Helvetica", 12)
    c.drawString(left_margin + 25, stub_2_y, payee_name)
    stub_2_y -= stub_2_y_offset * 1.5
    c.drawString(left_margin + 25, stub_2_y, memo)
    stub_2_y -= stub_2_y_offset
    if payor_account_number:
        c.drawString(left_margin + 25, stub_2_y, f"Account: {payor_account_number}")
    stub_2_y -= stub_2_y_offset
    if invoice:
        c.drawString(left_margin + 25, stub_2_y, f"Invoice: {invoice}")
        stub_2_y -= stub_2_y_offset
    # Add category if exists
    if hasattr(check, "category") and check.category:
        c.drawString(left_margin + 25, stub_2_y, f"Category: {check.category.name}")
        stub_2_y -= stub_2_y_offset
    # Add tags if exist
    if hasattr(check, "tags") and check.tags.exists():
        tags_str = ", ".join(tag.name for tag in check.tags.all())
        c.drawString(left_margin + 25, stub_2_y, f"Tags: {tags_str}")
        stub_2_y -= stub_2_y_offset
    c.drawString(
        left_margin + 25,
        stub_2_y,
        f"{bank_name} | Checking Account: {ACH_ACCOUNT_NUMBER}",
    )
    stub_2_y -= stub_2_y_offset
    c.drawString(left_margin + 25, stub_2_y, f"From: {payor_name}")
    c.drawString(left_margin + 450, stub_2_y, check_date)

    c.showPage()
    c.save()
    buf.seek(0)
    return buf.read()
    # All old code removed; only generate_check_pdf remains



from django.db import models
from django.core.files.base import ContentFile
from django.utils import timezone
from taggit.managers import TaggableManager

# Bank account model


class BankAccount(models.Model):
    account_nickname = models.CharField(max_length=255, blank=True)
    institution_name = models.CharField(max_length=255)
    street_1 = models.CharField(max_length=255, blank=True)
    street_2 = models.CharField(max_length=255, blank=True)
    city = models.CharField(max_length=255, blank=True)
    phone = models.CharField(max_length=50, blank=True)
    routing_number = models.CharField(max_length=20)
    account_number = models.CharField(max_length=20)
    payor_name = models.CharField(max_length=255, blank=True)
    payor_street_1 = models.CharField(max_length=255, blank=True)
    payor_street_2 = models.CharField(max_length=255, blank=True)
    payor_city = models.CharField(max_length=255, blank=True)
    payor_phone = models.CharField(max_length=50, blank=True)

    def save(self, *args, **kwargs):
        if not self.account_nickname:
            self.account_nickname = self.institution_name
        super().save(*args, **kwargs)

    def __str__(self):
        return (
            f"{self.account_nickname or self.institution_name} ({self.account_number})"
        )


# Payee model


class Payee(models.Model):
    name = models.CharField(max_length=255)
    street_1 = models.CharField(max_length=255, blank=True)
    street_2 = models.CharField(max_length=255, blank=True)
    city = models.CharField(max_length=255, blank=True)
    payor_account_number = models.CharField(max_length=20, blank=True)

    def __str__(self):
        return self.name


# Check model
class Check(models.Model):
    bank_account = models.ForeignKey(BankAccount, on_delete=models.CASCADE)
    payee = models.ForeignKey(Payee, on_delete=models.CASCADE)
    category = models.ForeignKey(
        "Category", on_delete=models.SET_NULL, blank=True, null=True
    )
    check_number = models.PositiveIntegerField(unique=True, editable=False)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    memo = models.CharField(max_length=255, blank=True)
    invoice = models.CharField(max_length=255, blank=True)
    check_date = models.DateField(default=timezone.now)
    pdf = models.FileField(upload_to="checks/", blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    tags = TaggableManager(blank=True)

    def __str__(self):
        return f"Check #{self.check_number} to {self.payee}"

    def save(self, *args, **kwargs):
        # Auto-generate unique check_number if not set
        if not self.check_number:
            last = Check.objects.order_by("-check_number").first()
            self.check_number = (last.check_number + 1) if last else 1
        super().save(*args, **kwargs)
        from .helpers import generate_check_pdf

        pdf_content = generate_check_pdf(self)
        filename = f"check_{self.pk}.pdf"
        self.pdf.save(filename, ContentFile(pdf_content), save=False)
        super().save(*args, **kwargs)

    @staticmethod
    def export_to_monarchmoney_csv(queryset):
        """
        Export selected checks as MonarchMoney-compatible CSV.
        Returns a tuple: (filename, csv_content)
        """
        import csv
        import io

        output = io.StringIO()
        writer = csv.writer(output)
        writer.writerow(
            [
                "Date",
                "Merchant",
                "Category",
                "Account",
                "Original Statement",
                "Notes",
                "Amount",
                "Tags",
            ]
        )
        for check in queryset:
            date = check.check_date.strftime("%Y-%m-%d")
            merchant = check.payee.name
            category = check.category.name if check.category else ""
            account = (
                check.bank_account.account_nickname
                or check.bank_account.institution_name
            )
            original_statement = (
                f"{check.payee.name} - {check.payee.payor_account_number}"
            )
            notes = f"{check.check_number} - {check.payee.name} - {check.payee.payor_account_number} - {check.memo} - {check.invoice}"
            amount = f"{check.amount:.2f}"
            tags = (
                ", ".join(tag.name for tag in check.tags.all())
                if hasattr(check, "tags")
                else ""
            )
            writer.writerow(
                [
                    date,
                    merchant,
                    category,
                    account,
                    original_statement,
                    notes,
                    amount,
                    tags,
                ]
            )
        filename = "checks_monarchmoney.csv"
        csv_content = output.getvalue()
        output.close()
        return filename, csv_content


# Category model
class Category(models.Model):
    name = models.CharField(max_length=100, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Categories"
        ordering = ["name"]
